import json
from django.contrib import admin
from django.contrib.admin.options import TO_FIELD_VAR
from django.template.response import TemplateResponse
from streamblocks.models import STREAMBLOCKS_MODELS

class StreamBlocksAdmin(admin.ModelAdmin):
    change_form_template = 'streamfield/admin/change_form.html'
    popup_response_template = 'streamfield/admin/streamfield_popup_response.html'

    def response_add(self, request, obj, post_url_continue=None):
        if "block_id" in request.POST:
            opts = obj._meta
            to_field = request.POST.get(TO_FIELD_VAR)
            attr = str(to_field) if to_field else opts.pk.attname
            value = obj.serializable_value(attr)
            popup_response_data = json.dumps({
                'app_id': request.POST.get("app_id"),
                'block_id': request.POST.get("block_id"),
                'instance_id': str(value),
            })
            return TemplateResponse(request, self.popup_response_template, {
                            'popup_response_data': popup_response_data,
                        })
        return super().response_add(request, obj, post_url_continue)

    def response_change(self, request, obj):
        if "block_id" in request.POST:
            opts = obj._meta
            to_field = request.POST.get(TO_FIELD_VAR)
            attr = str(to_field) if to_field else opts.pk.attname
            value = request.resolver_match.kwargs['object_id']
            new_value = obj.serializable_value(attr)
            popup_response_data = json.dumps({
                'action': 'change',
                'app_id': request.POST.get("app_id"),
                'block_id': request.POST.get("block_id"),
                'instance_id': request.POST.get("instance_id"),
            })
            return TemplateResponse(request, self.popup_response_template, {
                            'popup_response_data': popup_response_data,
                        })

        return super().response_change(request, obj)

    def response_delete(self, request, obj_display, obj_id):
        if "block_id" in request.POST:
            popup_response_data = json.dumps({
                'action': 'delete',
                'value': str(obj_id),
                'app_id': request.POST.get("app_id"),
                'block_id': request.POST.get("block_id"),
                'instance_id': request.POST.get("instance_id"),
            })
            return TemplateResponse(request, self.popup_response_template, {
                            'popup_response_data': popup_response_data,
                        })

        return super().response_delete(request, obj_display, obj_id)
    
# if user defined admin for his blocks, then do not autoregiser block models

for model in STREAMBLOCKS_MODELS:
    if not model._meta.abstract and \
    not admin.site.is_registered(model):
        admin.site.register(model, StreamBlocksAdmin)


# -*- coding: utf-8 -*-
from rest_framework import serializers

def get_serializer_class(model, base=serializers.ModelSerializer):
    model_ = model

    class Meta:
        model = model_
        fields = '__all__'

    attrs = dict(
        Meta = Meta,
        )

    return type(str(model.__name__ + 'Serializer'), (base, ), attrs)


# -*- coding: utf-8 -*-
import copy

from django.views.generic import DetailView, TemplateView

from rest_framework import viewsets

from .serializers import get_serializer_class
from .forms import get_form_class

def get_viewset_class(model, base=viewsets.ModelViewSet):
    attrs = dict(
        queryset = model.objects.all(),
        serializer_class = get_serializer_class(model)
        )

    return type(str(model.__name__ + 'ViewSet'), (base, ), attrs )


def admin_instance_class(model, base=DetailView):
    tmpl = 'streamfield/admin/change_form_render_template.html'

    if hasattr(model, 'custom_admin_template'):
        tmpl = model.custom_admin_template

    def get_context_data(self, **kwargs):
        context = base.get_context_data(self, **kwargs)
        
        # forms
        obj = super(self.__class__, self).get_object()
        context['form'] = get_form_class(model)(instance=obj)

        return context

    attrs = dict(
        model = model,
        template_name = tmpl,
        get_context_data = get_context_data
        )

    return type(str(model.__name__ + 'DetailView'), (base, ), attrs )



from django.urls import include, path

from . import views
from streamblocks.models import STREAMBLOCKS_MODELS

admin_instance_urls = []

for model in STREAMBLOCKS_MODELS:
    if not model._meta.abstract:
        admin_instance_urls.append(path(
                    'admin-instance/%s/<int:pk>' % model.__name__.lower(), 
                    views.admin_instance_class(model).as_view(),
                    name='admin-instance'
                    ))

urlpatterns = [
    *admin_instance_urls
]



from django import template
from django.utils.text import (
    get_valid_filename, 
    camel_case_to_spaces
    )
from django.utils.safestring import mark_safe
from django.template import loader

register = template.Library()


@register.simple_tag
def format_field(field):
    widget_name = get_widget_name(field)

    try:
        t = loader.get_template('streamfield/admin/fields/%s.html' % widget_name)
    except loader.TemplateDoesNotExist:
        t = loader.get_template('streamfield/admin/fields/default.html')

    if widget_name == 'select':
        
        # ForeignKey Field
        if hasattr(field.field, '_queryset'):
            for obj in field.field._queryset:
                if obj.pk == field.value():
                    field.obj = obj

        # CharField choices
        if hasattr(field.field, '_choices'):
            for obj in field.field._choices:
                if obj[0] == field.value():
                    field.obj = obj[1]
        

    return mark_safe(t.render(dict(
        field=field
        )))


def get_widget_name(field):
    return get_valid_filename(
                camel_case_to_spaces(field.field.widget.__class__.__name__)
                )